/* FromToModel.java

	Purpose:
		
	Description:
		
	History:
		Tue Dec 10 15:15:35 CST 2019, Created by rudyhuang

Copyright (C) 2019 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart.model;

import java.util.Collection;

/**
 * A (from, to) chart data model.
 *
 * @author rudyhuang
 * @since 7.2.1.0
 */	
public interface FromToModel extends ChartsModel {
	/**
	 * Get a series of the specified index;
	 */
	Comparable<?> getSeries(int index);
	
	/**
	 * Get all series as a collection.
	 */
	Collection<Comparable<?>> getSeries();
	
	/**
	 * Get data count of a specified series.
	 * @param series the specified series.
	 */
	int getDataCount(Comparable<?> series);

	/**
	 * Get from value of a specified series and data index.
	 * @param series the series.
	 * @param index the data index.
	 */
	String getFrom(Comparable<?> series, int index);

	/**
	 * Get to value of a specified series and data index.
	 * @param series the series.
	 * @param index the data index.
	 */
	String getTo(Comparable<?> series, int index);

	/**
	 * Replace the value of the new (from, to) into a series at specified index.
	 * @param series the series.
	 * @param from the from value.
	 * @param to the to value.
	 * @param index the data index.
	 */
	void setValue(Comparable<?> series, String from, String to, int index);
	
	/**
	 * Append a (from, to) into a series.
	 * @param series the series.
	 * @param from the from value.
	 * @param to the to value.
	 */
	void addValue(Comparable<?> series, String from, String to);

	/**
	 * Add a (from, to) into a series at specified index.
	 * @param series the series.
	 * @param from the from value.
	 * @param to the to value.
	 * @param index the data index.
	 */
	void addValue(Comparable<?> series, String from, String to, int index);
	
	/**
	 * Set model to autosort on x value for each series.
	 * @deprecated since 7.2.2.0. Added by mistake
	 */
	@Deprecated
	void setAutoSort(boolean auto);

	/**
	 * check whether to autosort on x value for each series; default is true.
	 * @deprecated since 7.2.2.0. Added by mistake
	 */
	@Deprecated
	boolean isAutoSort();
	
	/**
	/**
	 * Remove data of a specified series.
	 * @param series the series
	 */
	void removeSeries(Comparable<?> series);

	/**
	 * Remove (from, to) value of a specified series and data index.
	 * @param series the series.
	 * @param index the data index.
	 */
	void removeValue(Comparable<?> series, int index);

	/**
	 * clear this model.
	 */
	void clear();
}	
