/* BellCurvePlotOptions.java

	Purpose:
		
	Description:
		
	History:
		Wed Nov 20 10:59:12 CST 2019, Created by rudyhuang

Copyright (C) 2019 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart.plotOptions;

import org.zkoss.chart.PlotAttribute;
import org.zkoss.chart.util.DynamicalAttribute;

/**
 * An option for bell curve plot
 * <p>
 * All the options in this class support {@link DynamicalAttribute}.
 *
 * @author rudyhuang
 * @since 7.2.1.0
 */
public class BellCurvePlotOptions extends SeriesPlotOptions {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		intervals,
		pointsInInterval,
	}

	/**
	 * Returns the length of the bell curve.
	 * A unit of the length of the bell curve is standard deviation.
	 * <p>
	 * Default: 3
	 * @since 7.2.1.0
	 */
	public Number getIntervals() {
		return getAttr(Attrs.intervals, 3).asNumber();
	}

	/**
	 * Sets the length of the bell curve.
	 * A unit of the length of the bell curve is standard deviation.
	 * @since 7.2.1.0
	 */
	public void setIntervals(Number intervals) {
		setAttr(Attrs.intervals, intervals, 3);
	}

	/**
	 * Returns how many points should be plotted within 1 interval.
	 * See {@link #getIntervals()}.
	 * <p>
	 * Default: 3
	 * @since 7.2.1.0
	 */
	public Number getPointsInInterval() {
		return getAttr(Attrs.pointsInInterval, 3).asNumber();
	}

	/**
	 * Sets how many points should be plotted within 1 interval.
	 * See {@link #getIntervals()}.
	 * @since 7.2.1.0
	 */
	public void setPointsInInterval(Number pointsInInterval) {
		setAttr(Attrs.pointsInInterval, pointsInInterval, 3);
	}
}
